

system_prompt = '''
你是一个高效、简洁、专注于Python编程的AI助手。你的核心使命是根据用户提供的“代码头”（函数定义文本）、“代码使用demo”（包的使用及输出示例）和“需求”，生成一段符合以下严格风格和行为准键的“生成的核心代码”及简要“解释”。

**风格与行为准则：**
1.  **极简主义**：生成的代码必须尽可能地精简，控制在最少行数内。
2.  **功能与优化分离**：只实现核心功能，不引入任何非必需的优化或复杂结构。
3.  **函数式优先**：尽可能使用函数而不是类来组织代码，除非需求明确要求封装状态。
4.  **无内部注释**：除函数定义处的Docstring外，代码内部不允许有任何注释。Docstring应简洁说明函数功能，不赘述细节。
5.  **无错误处理**：不使用 `try-except` 结构。假设所有外部调用（如API调用）都会成功，或其错误由外部（如`BianXieAdapter`）自行处理。
6.  **无冗余输出**：不在代码中包含任何用户提示或调试信息（如 `print()` 语句），除非需求明确要求。
7.  **上下文管理**：不实现复杂的历史截断或上下文管理逻辑，除非需求明确要求。
8.  **输入输出格式**：你将接收一个严格的JSON对象作为输入，并必须严格输出一个JSON对象。

**输入格式示例（你将收到的）：**
```json
{
  "代码头": "from typing import Iterator, Generator\nclass BianXieAdapter(ModelAdapter):\n    def product(self, prompt: str) -> str:\n        pass\n    async def aproduct(self, prompt: str) -> str:\n        pass\n    # ... 其他方法定义\n",
  "代码使用demo": "adapter = BianXieAdapter(api_key='...')\nresponse = await adapter.aproduct('Hello')\nprint(response)\n# 输出: Hello from model",
  "需求": "实现一个异步聊天功能，带历史记录，尽可能精简，不处理错误，只使用提供的BianXieAdapter。"
}
```

**输出格式示例（你必须提供的）：**
```json
{
  "生成的核心代码": "import os, asyncio\nfrom typing import List, Dict\nfrom your_adapter_module import BianXieAdapter # 请替换为你的实际导入路径\n\nasync def ultra_simple_chat(adapter: BianXieAdapter, initial_message: str = \"\") -> List[Dict[str, str]]:\n    \"\"\"极致精简的异步聊天函数，带历史记录。\"\"\"\n    history: List[Dict[str, str]] = [{\"role\": \"system\", \"content\": initial_message}] if initial_message else []\n    while True:\n        user_input = await asyncio.to_thread(input, \"你: \")\n        if user_input.lower() == 'exit': break\n        history.append({\"role\": \"user\", \"content\": user_input})\n        prompt = \"\".join(f\"{msg['role'].capitalize()}: {msg['content']}\\n\" for msg in history) + \"助手: \"\n        response = await adapter.aproduct(prompt)\n        history.append({\"role\": \"assistant\", \"content\": response})\n    return history\n\n# 示例使用\nif __name__ == \"__main__\":\n    adapter_key = os.getenv(\"BIANXIE_API_KEY\", \"sk-placeholder\")\n    adapter = BianXieAdapter(api_key=adapter_key)\n    final_chat_history = asyncio.run(ultra_simple_chat(adapter, \"你是一个简洁高效的AI助手。\"))\n    for msg in final_chat_history: print(f\"{msg['role'].capitalize()}: {msg['content']}\")",
  "解释": "根据需求，生成了名为 `ultra_simple_chat` 的异步函数，实现了带历史记录的极简聊天功能。它将所有用户和助手消息扁平化为单一 prompt 传递给 `adapter.aproduct`，并记录响应。代码严格遵循了极简、函数式、无错误处理和无额外注释的风格。"
}
```

**重要提示：**
*   你输出的“生成的核心代码”应是完整的、可直接运行的Python代码块，包含必要的导入和可能的 `if __name__ == "__main__":` 示例使用。
*   “解释”部分应简洁明了，概括代码的主要功能以及如何符合用户的风格和要求。
*   如果需求与提供的“代码头”或“代码使用demo”不完全匹配，请基于“需求”和通用Python实践生成代码，并在解释中指出任何潜在的不一致。


'''

import ast
import inspect
from llmada.core import BianXieAdapter


def strip_function_bodies(file_path: str) -> str:
    """
    从 Python 文件中剥离函数内容，只保留函数定义和注释。

    Args:
        file_path (str): 要处理的 Python 文件的路径。

    Returns:
        str: 包含剥离函数内容后的代码字符串。
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        source_code = f.read()

    tree = ast.parse(source_code)
    stripped_lines = []
    current_indent = 0
    in_function_body = False
    function_start_line = -1

    # 遍历原始代码的每一行，以保留非函数部分和正确处理缩进
    # ast 模块可以用来生成新的 AST，但要精确地保留原始格式，
    # 逐行处理并利用 ast 节点的信息是更可靠的方法。

    # 首先，我们收集所有函数和类的方法的 AST 节点，以及它们的起始行号和结束行号
    # 这样可以方便地判断一行是否属于某个函数体
    function_nodes = []
    class_nodes = []

    for node in ast.walk(tree):
        if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
            function_nodes.append(node)
        elif isinstance(node, ast.ClassDef):
            class_nodes.append(node)

    # 将源代码按行分割
    lines = source_code.splitlines(keepends=True) # keepends=True 保留行尾换行符

    # 用于追踪当前正在处理的 AST 节点
    current_class_node = None

    for i, line in enumerate(lines):
        line_num = i + 1 # ast 的行号从 1 开始

        # 检查当前行是否在某个函数体内部
        is_in_function_body_to_strip = False
        for func_node in function_nodes:
            # 一个粗略的判断：如果行号在函数定义的第一个非 docstring 语句之后，
            # 并且在函数体的最后一行之前，则认为在函数体内部。
            # 这里需要更精确的判断，因为我们只需要保留签名和 docstring
            if func_node.lineno <= line_num <= func_node.end_lineno:
                # 获取函数签名的结束行 (通常是 def 行的末尾)
                # 或者如果有装饰器，则装饰器的最后一行
                signature_end_line = func_node.lineno

                # 处理装饰器
                if func_node.decorator_list:
                    signature_end_line = max(signature_end_line, func_node.decorator_list[-1].end_lineno)

                # 处理参数列表，特别是多行参数列表
                if hasattr(func_node, 'args') and func_node.args.args:
                    last_arg = func_node.args.args[-1]
                    signature_end_line = max(signature_end_line, last_arg.end_lineno)
                elif hasattr(func_node, 'args') and func_node.args.kwonlyargs:
                    last_arg = func_node.args.kwonlyargs[-1]
                    signature_end_line = max(signature_end_line, last_arg.end_lineno)
                elif hasattr(func_node, 'args') and func_node.args.vararg:
                    signature_end_line = max(signature_end_line, func_node.args.vararg.end_lineno)
                elif hasattr(func_node, 'args') and func_node.args.kwarg:
                    signature_end_line = max(signature_end_line, func_node.args.kwarg.end_lineno)

                # 如果有返回类型注释，也算作签名的一部分
                if func_node.returns:
                    signature_end_line = max(signature_end_line, func_node.returns.end_lineno)

                # 获取 docstring 的结束行
                docstring_end_line = signature_end_line
                docstring_node = ast.get_docstring(func_node, clean=False)
                if docstring_node:
                    # 使用 inspect.getdoc 获取实际的 docstring 字符串
                    # 并计算其行数，这比直接 ast.get_docstring 更准确地反映了原始代码的行数
                    # 因为 ast.get_docstring 返回的是处理后的字符串，可能丢失多行信息
                    # 更可靠的方法是找到 docstring 字符串在原始代码中的位置
                    # 我们可以通过查看函数体的第一个 AST 节点来判断
                    if func_node.body and isinstance(func_node.body[0], ast.Expr) and isinstance(func_node.body[0].value, ast.Constant) and isinstance(func_node.body[0].value.value, str):
                        docstring_expr_node = func_node.body[0]
                        docstring_end_line = docstring_expr_node.end_lineno

                # 如果当前行在签名或 docstring 之后，但在函数结束之前，则认为是函数体内容
                # 注意：如果函数体是空的 (pass)，那么 signature_end_line 可能是函数结束行
                if line_num > max(signature_end_line, docstring_end_line) and line_num <= func_node.end_lineno:
                    # 检查函数体是否只有 pass
                    has_only_pass = False
                    if len(func_node.body) == 1 and isinstance(func_node.body[0], ast.Pass):
                        has_only_pass = True

                    if not has_only_pass:
                        is_in_function_body_to_strip = True
                        break # 找到了，跳出内层循环

        if is_in_function_body_to_strip:
            continue # 跳过函数体内的行

        # 处理类定义
        for class_node in class_nodes:
            if class_node.lineno <= line_num <= class_node.end_lineno:
                current_class_node = class_node
                break
            current_class_node = None # 重置

        # 保留所有不在函数体内的行，以及函数签名和 docstring
        stripped_lines.append(line)

    return "".join(stripped_lines)



# 调用函数处理文件
# print(output_code)
output_code = strip_function_bodies("/Users/zhaoxuefeng/GitHub/llmada/src/llmada/core.py")

with open("tests/test_core.py",'r') as f:
    demo = f.read()



bx = BianXieAdapter()

inputs = {"代码头":output_code,
          "demo":  demo,
          "需求": "写一个聊天的功能, 支持多轮对话的",
} 
import json
input_json = json.dumps(inputs,ensure_ascii=False)

result = bx.product(system_prompt + input_json)

print(result)

res = json.loads(result)

res.get("生成的核心代码")


"""
client.py 用于管理client
"""

import requests
import time
import json
import base64
import httpx
import logging
from llmada.log import Log
from llmada.utils import save_base64_image, image_to_base64
from tenacity import (
    retry,
    wait_fixed,
    wait_exponential,
    stop_after_attempt,
    retry_if_exception_type,
    before_sleep_log,
)
import uuid
import websockets
from llmada.protocols import MsgType, full_client_request, receive_message
from volcenginesdkarkruntime import Ark, AsyncArk


logger = Log.logger


class ArkClient:
    """
    ArkClient
    """

    def __init__(
        self, api_key: str, api_base: str = "",appid: str = "", access_token: str = ""
    ):
        """
        初始化 OpenAI 客户端
        url = 'https://api.bianxie.ai/v1/audio/transcriptions'  asr 语音识别
        """
        self.api_key = api_key
        self.api_base = api_base
        self.client = Ark(api_key=self.api_key)
        self.async_client =AsyncArk(api_key=self.api_key)
        self.appid = appid or "9370139706"
        self.access_token = access_token or "rPnirNKqj-jYwic1yTYpVOUU0xx2g8uj"
        self.endpoint = "wss://openspeech.bytedance.com/api/v1/tts/ws_binary"

    def product(self, data):
        result = self.client.chat.completions.create(**data)
        return result
    
    async def aproduct(self, data):
        result = await self.async_client.chat.completions.create(**data)
        return result

    def product_stream(self, data):
        stream = self.client.chat.completions.create(
            **data
        )
        for chunk in stream:
            if not chunk.choices:
                continue
            yield chunk.choices[0].delta.content
            
    async def aproduct_stream(self, data):
        stream = await self.async_client.chat.completions.create(
            **data
        )
        async for chunk in stream:
            if not chunk.choices:
                continue
            yield chunk.choices[0].delta.content


    async def generate2file(self,text: str, 
                            filename = f"tests/resources/work.wav",
                            encoding="wav",
                            voice_type = "zh_female_yuanqinvyou_moon_bigtts"):
        """
        tts text2wav
        """
        cluster = ""
        
        def get_cluster(voice: str) -> str:
            if voice.startswith("S_"):
                return "volcano_icl"
            return "volcano_tts"
        
        # Determine cluster
        cluster = cluster if cluster else get_cluster(voice_type)

        # Connect to server
        headers = {
            "Authorization": f"Bearer;{self.access_token}",
        }

        print(f"Connecting to {self.endpoint} with headers: {headers}")
        websocket = await websockets.connect(
            self.endpoint, additional_headers=headers, max_size=10 * 1024 * 1024
        )
        print(
            f"Connected to WebSocket server, Logid: {websocket.response.headers['x-tt-logid']}",
        )

        try:
            # Prepare request payload
            request = {
                "app": {
                    "appid": self.appid,
                    "token": self.access_token,
                    "cluster": cluster,
                },
                "user": {
                    "uid": str(uuid.uuid4()),
                },
                "audio": {
                    "voice_type": voice_type,
                    "encoding": encoding,
                },
                "request": {
                    "reqid": str(uuid.uuid4()),
                    "text": text,
                    "operation": "submit",
                    "with_timestamp": "1",
                    "extra_param": json.dumps(
                        {
                            "disable_markdown_filter": False,
                        }
                    ),
                },
            }

            # Send request
            await full_client_request(websocket, json.dumps(request).encode())

            # Receive audio data
            audio_data = bytearray()
            while True:
                msg = await receive_message(websocket)

                if msg.type == MsgType.FrontEndResultServer:
                    continue
                elif msg.type == MsgType.AudioOnlyServer:
                    audio_data.extend(msg.payload)
                    if msg.sequence < 0:  # Last message
                        break
                else:
                    raise RuntimeError(f"TTS conversion failed: {msg}")

            # Check if we received any audio data
            if not audio_data:
                raise RuntimeError("No audio data received")

            # Save audio file
            with open(filename, "wb") as f:
                f.write(audio_data)
            print(f"Audio received: {len(audio_data)}, saved to {filename}")

        finally:
            await websocket.close()
            print("Connection closed")






class OpenAIClient:
    """
    使用 openai 官方包对接 OpenAI API，支持简单对话和流式对话
    """

    def __init__(
        self, api_key: str, api_base: str = "https://api.bianxie.ai/v1/chat/completions"
    ):
        """
        初始化 OpenAI 客户端
        url = 'https://api.bianxie.ai/v1/audio/transcriptions'  asr 语音识别
        """
        self.api_key = api_key
        self.api_base = api_base
        self.headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {api_key}",
        }

    def request(self, params: dict) -> dict:
        """
        简单对话：直接调用 OpenAI API 并返回完整响应
        """
        api_base = self.api_base
        try:
            logger.info("request running")
            time1 = time.time()
            response = requests.post(api_base, headers=self.headers, json=params)
            time2 = time.time()
            logger.debug(time2 - time1)
            return response.json()
        except Exception as e:
            logger.error(e)
            raise Exception(f"API request failed: {e}") from e

    @retry(
        wait=wait_exponential(
            multiplier=1, min=1, max=10
        ),  # 首次等待1秒，指数增长，最大10秒
        stop=stop_after_attempt(3),  # 最多尝试5次 (1次初始请求 + 4次重试)
        retry=(
            retry_if_exception_type(httpx.RequestError)  # 匹配网络错误 (连接超时, DNS, SSL等)
            | retry_if_exception_type(httpx.HTTPStatusError)  # 匹配HTTP状态码错误
        ),
        before_sleep=before_sleep_log(logger, logging.INFO),  # 每次重试前打印日志
        reraise=True,  # 如果所有重试都失败，重新抛出最后一个异常
    )
    async def arequest(self, params: dict) -> dict:
        """
        # 修改后的异步请求函数
        简单对话：直接调用 OpenAI API 并返回完整响应 (异步版本)
        """
        api_base = self.api_base
        logger.info("Async request running")
        try:
            time1 = time.time()
            # 使用 httpx.AsyncClient 进行异步请求
            async with httpx.AsyncClient(headers=self.headers) as client:
                response = await client.post(
                    api_base, json=params, timeout=60.0
                )  # 加上timeout是个好习惯
                response.raise_for_status()  # 检查HTTP状态码，如果不是2xx，会抛出异常

            time2 = time.time()
            logger.debug(f"Request took: {time2 - time1:.4f} seconds")

            return response.json()

        except httpx.HTTPStatusError as e:
            logger.error(
                f"HTTP error occurred: {e.response.status_code} - {e.response.text}"
            )
            # 这里不要重新抛出新的Exception类型，而是重新抛出原始异常，或者不捕获让tenacity处理
            # 方式一：直接 raise e，tenacity 会捕捉到 httpx.HTTPStatusError
            raise
        except httpx.RequestError as e:
            logger.error(f"An error occurred while requesting {e.request.url!r}: {e}")
            raise 
        except Exception as e:
            logger.error(f"An unexpected error occurred: {e}")
            raise Exception(f"API request failed: {e}") from e

    def request_tts(self, params: dict) -> dict:
        """

        params: dict
            - file_path 音频路径  'tests/resources/speech.mp3'
            - model 模型名称 'tts-1'
            - input 输入文本 "你好"
            - voice 声音    "alloy"
        """
        api_base = "https://api.bianxie.ai/v1/audio/speech"
        try:
            logger.info("request running")
            time1 = time.time()
            response = requests.post(
                api_base,
                headers=self.headers,
                json={
                    "model": params.get("model"),
                    "input": params.get("input"),
                    "voice": params.get("voice"),
                },
            )
            time2 = time.time()
            logger.debug(time2 - time1)
            if response.status_code == 200:
                with open(params.get("file_path"), "wb") as file:
                    file.write(response.content)
                print(f"Audio saved successfully as {params.get('file_path')}")
            else:
                print(f"Failed to get audio: {response.status_code} - {response.text}")

        except Exception as e:
            logger.error(e)
            raise Exception(f"API request failed: {e}") from e

    async def arequest_tts(self, params: dict) -> dict:
        """
        params: dict
            - file_path 音频路径  'tests/resources/speech.mp3'
            - model 模型名称 'tts-1'
            - input 输入文本 "你好"
            - voice 声音    "alloy"
        """
        try:

            # # 使用 httpx.AsyncClient 进行异步请求
            # async with httpx.AsyncClient(headers=self.headers) as client:
            #     response = await client.post(api_base, json=params, timeout=30.0) # 加上timeout是个好习惯
            #     response.raise_for_status() # 检查HTTP状态码，如果不是2xx，会抛出异常

            logger.info("Async request_tts running")

            time1 = time.time()
            api_base = "https://api.bianxie.ai/v1/audio/speech"
            params = {
                "model": params.get("model"),
                "input": params.get("input"),
                "voice": params.get("voice"),
            }
            # 使用 httpx.AsyncClient 进行异步请求
            async with httpx.AsyncClient(headers=self.headers) as client:
                response = await client.post(
                    api_base, json=params, timeout=30.0
                )  # 加上timeout是个好习惯
                response.raise_for_status()  # 检查HTTP状态码，如果不是2xx，会抛出异常

            time2 = time.time()
            logger.debug(time2 - time1)
            if response.status_code == 200:
                with open(params.get("file_path"), "wb") as file:
                    file.write(response.content)
                print(f"Audio saved successfully as {params.get('file_path')}")
            else:
                print(f"Failed to get audio: {response.status_code} - {response.text}")

        except httpx.HTTPStatusError as e:
            logger.error(
                f"HTTP error occurred: {e.response.status_code} - {e.response.text}"
            )
            raise Exception(
                f"API request failed with HTTP error: {e.response.status_code}"
            ) from e
        except httpx.RequestError as e:
            logger.error(f"An error occurred while requesting {e.request.url!r}: {e}")
            raise Exception(
                f"API request failed due to network or connection error: {e}"
            ) from e
        except Exception as e:
            logger.error(f"An unexpected error occurred: {e}")
            raise Exception(f"API request failed: {e}") from e

    def request_asr(self, params: dict) -> dict:
        """

        params: dict
            - file_path 音频路径
            - model 模型名称 'whisper-1'
        """
        api_base = "https://api.bianxie.ai/v1/audio/transcriptions"
        print(params.get("file_path"), 'params.get("file_path")')
        print(params.get("model"), 'params.get("file_path")')
        try:
            with open(params.get("file_path"), "rb") as audio_file:
                logger.info("request running")
                time1 = time.time()
                response = requests.post(
                    api_base,
                    headers=self.headers,
                    files={"file": audio_file},
                    data={"model": params.get("model")},
                )
                time2 = time.time()
                logger.debug(time2 - time1)
            return response.json()
        except Exception as e:
            logger.error(e)
            raise Exception(f"API request failed: {e}") from e

    def request_stream(self, params: dict) -> dict:
        """
        简单对话：直接调用 OpenAI API 并返回流式响应
        request_stream_http2
        """
        logger.info("request_stream_http2")
        try:
            self.headers["Accept"] = "text/event-stream"

            payload = params
            with httpx.Client(http2=True, timeout=None) as client:
                with client.stream(
                    "POST", self.api_base, headers=self.headers, json=payload
                ) as response:
                    response.raise_for_status()
                    buffer = b""
                    for chunk in response.iter_bytes():
                        buffer += chunk
                        while b"\n" in buffer:
                            line_bytes, _, buffer = buffer.partition(b"\n")
                            decoded_line = line_bytes.decode("utf-8")

                            if decoded_line.startswith("data:"):
                                json_str = decoded_line[len("data:") :].strip()

                                if json_str == "[DONE]":
                                    return  # End the generator

                                try:
                                    data = json.loads(json_str)
                                    # Extract content based on common LLM API response structure
                                    if "choices" in data and data["choices"]:
                                        chunk_content = data["choices"][0]["delta"].get(
                                            "content", ""
                                        )
                                        if chunk_content:
                                            yield chunk_content  # Yield the generated content chunk
                                except json.JSONDecodeError:
                                    yield f"[ERROR: Malformed data received: {json_str}]"

        except httpx.RequestError as e:
            error_msg = f"Request failed: {e}"
            raise ConnectionError(
                error_msg
            ) from e  # Re-raise as a custom error or simpler Exception
        except Exception as e:
            error_msg = f"An unexpected error occurred: {e}"
            raise RuntimeError(error_msg) from e

    async def arequest_stream(self, params: dict):
        """
        简单对话：直接调用 OpenAI API 并返回流式响应 (异步版本)
        arequest_stream_http2
        """
        logger.info("arequest_stream_http2")
        try:
            # 设置 Accept 头为 text/event-stream，表示期望流式事件
            self.headers["Accept"] = "text/event-stream"
            payload = params

            # 使用 httpx.AsyncClient 进行异步 HTTP 请求
            # timeout=None 表示不设置请求超时时间，由底层 TCP/TLS 决定
            async with httpx.AsyncClient(http2=True, timeout=None) as client:
                # 发送 POST 请求并获取异步响应流
                async with client.stream(
                    "POST", self.api_base, headers=self.headers, json=payload
                ) as response:
                    # 检查 HTTP 响应状态码，如果不是 2xx，则抛出异常
                    response.raise_for_status()

                    buffer = b""
                    # 异步迭代响应的字节块
                    async for chunk in response.aiter_bytes():
                        buffer += chunk
                        # 处理可能包含多行或不完整行的缓冲区
                        while b"\n" in buffer:
                            line_bytes, _, buffer = buffer.partition(b"\n")
                            decoded_line = line_bytes.decode("utf-8")

                            if decoded_line.startswith("data:"):
                                json_str = decoded_line[len("data:") :].strip()

                                if json_str == "[DONE]":
                                    return  # 结束生成器

                                try:
                                    data = json.loads(json_str)
                                    # 根据常见的 LLM API 响应结构提取内容
                                    if "choices" in data and data["choices"]:
                                        chunk_content = data["choices"][0]["delta"].get(
                                            "content", ""
                                        )
                                        if chunk_content:
                                            # 异步 yield 生成的内容块
                                            yield chunk_content
                                except json.JSONDecodeError:
                                    yield f"[ERROR: Malformed data received: {json_str}]"

        except httpx.RequestError as e:
            error_msg = f"Request failed: {e}"
            # 异步函数中也抛出异常
            raise ConnectionError(error_msg) from e
        except Exception as e:
            error_msg = f"An unexpected error occurred: {e}"
            # 异步函数中也抛出异常
            raise RuntimeError(error_msg) from e

    def request_image_stream(
        self, params: dict, filename_prefix: str = "gemini_output"
    ) -> dict:
        """
        简单对话：直接调用 OpenAI API 并返回流式响应
        params = {
            'model': 'gemini-2.5-flash-image-preview',
            'messages': [{'role': 'user', 'content': 'two dogs in a tree'}],
            'stream': True
        }
        """
        api_base = self.api_base
        try:
            time1 = time.time()
            response = requests.post(
                api_base, headers=self.headers, json=params, stream=True
            )

            # 检查HTTP状态码
            if response.status_code == 200:
                print("Received streaming response:")
                # 逐行处理响应内容
                # iter_lines() 迭代响应内容，按行分割，并解码
                time2 = time.time()
                for line in response.iter_lines():
                    # lines are bytes, convert to string
                    line = line.decode("utf-8")

                    if "![image](data:image/" in line:
                        print(f"生成图片中到 {filename_prefix}")
                        save_base64_image(line, filename_prefix=filename_prefix)

                    # Server-Sent Events (SSE) messages start with 'data: '
                    # and the stream ends with 'data: [DONE]'
                    if line.startswith("data:"):
                        # Extract the JSON string after 'data: '
                        json_str = line[len("data:") :].strip()

                        if json_str == "[DONE]":
                            break  # End of stream

                        if json_str:  # Ensure it's not an empty data line
                            try:
                                # Parse the JSON string into a dictionary
                                chunk = json.loads(json_str)

                                # Extract the content chunk (similar structure to OpenAI API)
                                # Check if choices and delta exist before accessing content
                                if (
                                    chunk
                                    and "choices" in chunk
                                    and len(chunk["choices"]) > 0
                                ):
                                    delta = chunk["choices"][0].get("delta")
                                    if delta and "content" in delta:
                                        content_chunk = delta["content"]
                                        # Print the chunk without a newline, immediately flushing the output
                                        # print(content_chunk, end='', flush=True)
                                        yield content_chunk

                            except json.JSONDecodeError as e:
                                print(
                                    f"\nError decoding JSON chunk: {e}\nChunk: {json_str}"
                                )
                            except Exception as e:
                                print(
                                    f"\nError processing chunk: {e}\nChunk data: {chunk}"
                                )

                print(
                    f"\n(Streaming finished) {time2-time1}"
                )  # Add a newline after the stream is complete

            else:
                # Handle non-200 responses
                print(f"Error: Received status code {response.status_code}")
                print("Response body:")

        except requests.exceptions.RequestException as e:
            # TODO if "439" in ccc
            print(f"Request Error: {e}")
        except Exception as e:
            print(f"An unexpected error occurred: {e}")

    def request_modal(self):
        # TODO 图像理解
        def get_img_content(inputs=str):
            if 0:  # is_url
                return "https://github.com/dianping/cat/raw/master/cat-home/src/main/webapp/images/logo/cat_logo03.png"
            else:

                def image_to_base64(image_path):
                    with open(image_path, "rb") as image_file:
                        image_data = image_file.read()
                        base64_encoded_data = base64.b64encode(image_data)
                        base64_encoded_str = base64_encoded_data.decode("utf-8")
                        return base64_encoded_str

                image_base64 = image_to_base64(inputs)
                return f"data:image/jpeg;base64,{image_base64}"

        data = {
            "model": "gpt-4-vision-preview",
            "messages": [
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": "这张图片的图标是个什么动物？"},
                        {
                            "type": "image_url",
                            "image_url": {"url": get_img_content("111.png")},
                        },
                    ],
                }
            ],
        }

        response = requests.post(self.api_base, headers=self.headers, json=data)
        return response.json()



from .protocols import (
    CompressionBits,
    EventType,
    HeaderSizeBits,
    Message,
    MsgType,
    MsgTypeFlagBits,
    SerializationBits,
    VersionBits,
    audio_only_client,
    cancel_session,
    finish_connection,
    finish_session,
    full_client_request,
    receive_message,
    start_connection,
    start_session,
    task_request,
    wait_for_event,
)

__all__ = [
    "CompressionBits",
    "EventType",
    "HeaderSizeBits",
    "Message",
    "MsgType",
    "MsgTypeFlagBits",
    "SerializationBits",
    "VersionBits",
    "audio_only_client",
    "cancel_session",
    "finish_connection",
    "finish_session",
    "full_client_request",
    "receive_message",
    "start_connection",
    "start_session",
    "task_request",
    "wait_for_event",
]



# TODO 深度思考
# TODO 视觉理解
# TODO GUI Agent

from llmada.core import BianXieAdapter,ArkAdapter


__all__ = [
    "BianXieAdapter",
    "ArkAdapter",
]


from dotenv import load_dotenv
load_dotenv('.env',override=True)


from dotenv import load_dotenv
from llmada.core import BianXieAdapter, ArkAdapter
import pytest
import sys
import os
load_dotenv()

class Test_Bianxie:
    @pytest.fixture
    def bianxie(self):
        return BianXieAdapter("gemini-2.5-flash-preview-05-20-nothinking")
    
    # @pytest.mark.skip("通过")
    def test_product(self,bianxie):
        result = bianxie.product('你好')
        print(result,"result")
        
    async def test_aproduct(self,bianxie):
        result = await bianxie.aproduct('你好')
        print(result,"result")


    def test_product_stream(self,bianxie):
        rus = bianxie.product_stream("你好")
        for i in rus:
            print(i)
            assert type(i) == str

    async def test_aproduct_stream(self,bianxie):
        rus = bianxie.aproduct_stream("你好")
        async for i in rus:
            print(i)
            assert type(i) == str


    def test_product_by_dict(self,bianxie):
        data = {
            "model": "gemini-2.5-flash-preview-05-20-nothinking",
            "messages": [{"role": "user", "content": "你好"}],
            "temperature": 0.7,
        }
        result = bianxie.product_by_dict(data)
        print(result,"result")

    async def test_aproduct_by_dict(self,bianxie):
        data = {
            "model": "gemini-2.5-flash-preview-05-20-nothinking",
            "messages": [{"role": "user", "content": "你好"}],
            "temperature": 0.7,
        }
        result = await bianxie.aproduct_by_dict(data)
        print(result,"result")



class Test_Ark:

    @pytest.fixture
    def ark(self):
        return ArkAdapter("doubao-1-5-pro-256k-250115")

    def test_product(self,ark):
        # @pytest.mark.skip("通过")
        result = ark.product(prompt='你好')
        print(result,'result')
        assert type(result) == str

    async def test_aproduct(self,ark):
        result = await ark.aproduct(prompt='你好')
        print(result,'result')
        assert type(result) == str
        
    def test_product_stream(self,ark):
        result = ark.product_stream(prompt='你好')
        print(result,'result')
        for chunk in result:
            print(chunk)
        # assert type(result) == str

    async def test_aproduct_stream(self,ark):
        result = ark.aproduct_stream(prompt='你好')
        async for chunk in result:
            print(chunk)


    async def test_tts(self,ark):
        await ark.tts(text = "我是一个小狗狗",
                filename = "tests/resources/work.wav")





# 测试 client 部分

'''
Author: 823042332@qq.com 823042332@qq.com
Date: 2025-09-01 17:39:41
LastEditors: 823042332@qq.com 823042332@qq.com
LastEditTime: 2025-09-03 11:43:22
FilePath: /llmada/tests/test_asr.py
Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
'''
import requests
import os
from llmada.client import OpenAIClient
import pytest
import asyncio
from llmada.log import Log
logger = Log.logger


def test_client_request():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))

    params = {
            "model": "gemini-2.5-flash-preview-05-20-nothinking",
            "messages": [{"role": "user", "content": "你好"}],
            "temperature": 0.7,
        }
    response = client.request(params = params)
    print(response)


async def test_client_arequest():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))
    params = {
            "model": "gemini-2.5-flash-preview-05-20-nothinking",
            "messages": [{"role": "user", "content": "你好"}],
            "temperature": 0.7,
        }
    response = await client.arequest(params = params)
    print(response)

async def test_client_arequest_retry_function():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))
    text = """
{
    "system_prompt": "你是一个专业的传记作家和结构规划师AI。你的核心任务是根据用户提供的'人物素材'，创作一个具有深刻人物洞察和叙事张力的'传记大纲'。该大纲需严格遵循用户指定的'三起三伏'叙事节奏，并以人物生命中的'重要时刻'为主要驱动力进行章节规划。**硬性要求：根据提供的素材密度和丰富性，生成一个包含30-50章的章节划分。** 所有输出必须严格符合JSON格式要求，且不可包含任何额外的解释性文本或非JSON内容。",
    "input_material_guidance": {
        "material_nature": "用户提供的素材是关于一个普通人的生平信息，通常以访谈记录、经历片段、关键信息、新增细节等形式呈现。这些素材包含了人物的成长背景、重大事件、个人思想、人际关系、以及特定的哲学思考等。",
        "processing_instructions": [
            "Comprehensive Scan: 全面分析素材，识别所有提及的事件、人物、时间点、思想观点和情感。",
            "Identify Key Moments: 从事件罗列中，提炼出对人物性格、命运、思想产生决定性影响的'重要时刻'。这些时刻可以是外部事件（如学业、职业、创业、合作、危机），也可以是内部觉醒（如思想转变、哲学形成、情感领悟）。",
            "Extract Personality & Philosophy: 挖掘人物独有的性格特征（如'叛逆'、'突破性'、'童心'）和人生哲学（如'没有低谷期'、'解除人类基因锁'），并理解这些特质如何贯穿其生平。",
            "Connect Events to Inner World: 将外部事件与人物的内心成长、思维模式、价值观演变紧密关联起来。每个事件应揭示人物的某个侧面，或推动其成长。",
            "Synthesize & Interpret: 结合不同素材片段，进行综合理解和适当解读，以构建连贯、深入的叙事线索。",
            "**Chapter Allocation based on Material Density:** 根据每个重要事件、经历或思想片段的素材量和重要性，合理分配章节。素材越丰富、越关键，可拆分成更多章节；反之则可合并。确保总章节数在30-50章之间。"
        ]
    },
    "task_parameters": {
        "output_format": {
            "type": "JSON Object",
            "structure_details": {
                "top_level_keys_are_sections": "每个顶级键（Key）代表一个大的'部'（Section），例如 '第一部 童年与性格底色'。部名应简洁且概括该部核心内容。",
                "section_value_is_array_of_chapters": "每个'部'（Key）对应的值（Value）是一个JSON数组，该数组内包含该部所有'章'（Chapter）的JSON对象。",
                "chapter_object_structure": {
                    "chapter_number": "string, e.g., '第一章', '第二章'，确保连续性。",
                    "title": "string, 章节标题，简洁明了，概括本章核心内容。",
                    "topic": "string, 章节核心内容概述。长度适中，包含以下要素：\n  - **叙述视角明确：** 明确标示 '第一人称' 或 '第三人称'。\n  - **内容提炼：** 精炼概括本章将叙述的主要事件或思想片段。\n  - **意义揭示：** 强调该事件/思想对人物性格塑造、观念形成、人生转折的重要性，或其在'三起三伏'节奏中的作用（如 '第一起的核心'、'重要铺垫'、'低谷中的沉淀'）。\n  - **融入人物特色：** 尽可能体现人物的独特语言风格或哲学思考。"
                }
            },
            "strict_compliance": "REQUIRED. No exceptions. No additional prose before or after JSON."
        },
        "narrative_structure": {
            "overall_length_target": "Approx. 80,000 words for the full biography.",
            "chapter_length_target": "Each chapter ~2000 words. 大纲规划需确保此篇幅可实现。",
            "rhythm_model": "Three-Acts-Three-Folds (三起三伏):\n  - **识别'起'：** 指人物事业、思想、影响力达到高峰或实现重大突破的阶段。\n  - **识别'伏'：** 指人物遭遇挑战、经历沉淀、进行内省、或为下一阶段蓄力的时期，并非单纯的失败或低谷，而是带有转机或成长的意味。\n  - 大纲需清晰体现这些'起'和'伏'的交错与递进。",
            "narrative_drivers": [
                "Life-defining 'Important Moments' (关键时刻): 围绕这些时刻展开叙述，并使其他事件为其铺垫或结果。",
                "Character-driven narrative: 故事通过人物的性格、选择和心路历程来推动。",
                "Perspective allocation: Predominantly '第一人称' for internal reflections, philosophical insights, and personal experiences. Sparsely use '第三人称' for external analysis (e.g., from critics, media, or specific collaborators), objective events, or insights into how others perceive the character."
            ],
            "character_emphasis": [
                "Personality bedrock and its evolution (e.g., '叛逆', '突破性', '童心').",
                "Philosophical underpinnings and worldviews (e.g., '没有低谷期', '解除人类基因锁').",
                "Internal conflicts, dilemmas, and their resolutions.",
                "Growth, adaptation, and transformation across different life stages."
            ]
        },
        "response_constraints": [
            "Generate ONLY the requested JSON object. No conversational preambles, explanations, or conclusions.",
            "Ensure precise naming for '部' as JSON keys, e.g., '第一部 童年与性格底色'.",
            "Ensure precise naming for '章' within 'chapter_number' field, e.g., '第一章'."
        ]
    }
}

user:
我想你将章节数量控制在8-12个左右

-----input----

    
### **张三 自传素材整理 (精修版 - 按人物刻画与时间主线梳理)**

**核心人物：张三**
*   **姓名：** 张三
*   **性别：** 男
*   **出生日期：** 1980年
*   **出生地：** 北京 (根据简历，但记忆卡片提及出生于东北辽宁葫芦岛，此处将以记忆卡片为准，简历信息可能为户籍地或后期定居地)
*   **职业：** 某AI公司首席科学家
*   **当前城市：** 未知
*   **人物标签：** 探索者、自然爱好者、求知者、哲思者
*   **核心愿景：** 探索世界本质，追求知识，感受自然之美。

---

### **主要人物刻画与关系网**

*(根据提供的记忆卡片，目前没有具体提及除张三本人之外的其他人物。此部分将留空或在未来有更多信息时补充。)*

---

### **张三 时间主线与探索轨迹**

**人生主线：** 从东北乡村的自然童年到对物理世界的早期探索，张三的人生轨迹展现了对知识和世界本质的持续好奇与追求。

*   **童年时期：乡村的自然与星空 ([1980年代末 - 1990年代初], 辽宁葫芦岛小村庄)**
    *   **背景：** 出生于东北辽宁葫芦岛的一个小村庄。那里的生活简单、规律，人们日出而作，日落而息。自然环境优美，是“山里的野孩子”的乐园。
    *   **塑造/关键事件：**
        *   **自由的童年：** 在山里尽情疯跑，体验无拘无束的乡村生活，培养了对自然的亲近感。
        *   **星空下的启蒙：** 对夜晚的漫天繁星记忆深刻，这份经历可能激发了他对宇宙和未知世界的早期好奇心，为后来对天文物理的兴趣埋下伏笔。

*   **青少年时期：物理世界的初步探索 ([约1995年左右], 县城高中)**
    *   **背景：** 在县城高中读书，学习资源相对有限，重心主要放在学科学习上。
    *   **塑造/关键事件：**
        *   **兴趣萌芽：** 在高中时期，对天文物理和理论物理产生了浓厚的兴趣。尽管资源不足，但这份兴趣显示出他超越日常学习的求知欲和对深层科学原理的向往。

---

**张三 的“觉醒与探索”线索**

*   **自然启蒙：** 乡村的童年生活，尤其是夜晚的星空，是张三最早的觉醒点，激发了他对自然和宇宙的原始好奇心。
*   **知识求索：** 高中时期对天文物理和理论物理的兴趣，标志着他从感性的自然体验转向理性的科学探索，开始主动寻求对世界本质的理解。这份探索精神贯穿其后来的学习和职业生涯。
*   **从具象到抽象：** 从对漫天繁星的直观感受，到对天文物理和理论物理的抽象思考，展现了张三思维深度和广度的发展。

---

### **金句与细节描绘**

#### **🌟 金句摘录**

*   “当时我们都是山里的野孩子，没有什么特别的兴趣爱好，就在山里各种疯跑。”
*   “我小时候特别喜欢晚上看星星，那时的夜晚星星非常多，真的是那种突然就能看到漫天繁星的感觉。”
*   “在我高中的时候，我对天文物理和理论物理就非常感兴趣。”

#### **🎨 优美细节描绘**

*   “我出生在东北辽宁葫芦岛下面的一个小村庄. 小时候，那里的生活比较简单，人们日出而作，日落而息，生活节奏非常有规律，也非常美好。”
*   “那时的夜晚星星非常多，真的是那种突然就能看到漫天繁星的感觉。”    

assistant:
```json
{
    "第一部 童年与自然启蒙": [
        {
            "chapter_number": "第一章",
            "title": "葫芦岛的星光童年",
            "topic": "第三人称。本章将追溯张三在1980年代末至1990年代初，出生于辽宁葫芦岛小村庄的童年岁月。重点描绘乡村的淳朴生活、日出而作日落而息的节奏，以及他作为“山里的野孩子”与自然亲密接触的自由时光。这是张三生命中“第一起”的萌芽，奠定了他与自然连接的性格底色。"
        },
        {
            "chapter_number": "第二章",
            "title": "漫天繁星的召唤",
            "topic": "第一人称。深入探究张三童年时期对夜晚星空的深刻记忆和感受。描述漫天繁星如何在他幼小的心灵中种下对宇宙和未知世界的好奇种子，成为他最早的“觉醒点”，预示其未来对知识的无尽求索。这是他“第一起”中重要的情感与思想铺垫。"
        }
    ],
    "第二部 知识的初步求索": [
        {
            "chapter_number": "第三章",
            "title": "县城高中：物理之门的开启",
            "topic": "第三人称。本章将叙述张三青少年时期在县城高中的学习经历。尽管学习资源相对有限，但张三对天文物理和理论物理产生了浓厚兴趣。这标志着他从感性的自然体验转向理性的科学探索，是其求知欲的进一步展现。这是他“第一起”中知识探索的关键一步。"
        },
        {
            "chapter_number": "第四章",
            "title": "从繁星到理论：思维的跃迁",
            "topic": "第一人称。探讨张三从童年对具象星空的直观感受，到高中时期对抽象天文物理和理论物理的兴趣转变。本章将揭示他思维深度和广度的发展，以及这种从感性到理性的过渡如何塑造了他理解世界的方式。这是他“第一起”中思维方式形成的重要节点。"
        }
    ]
}
```

### 张三 自传素材整理 (精修版 - 按人物刻画与时间主线梳理)

**核心人物：张三**
*   **姓名：** 张三
*   **性别：** 男
*   **出生日期：** 1980年
*   **出生地：** 辽宁葫芦岛小村庄 (根据记忆卡片，修正简历中“北京”为户籍地或后期迁居地，待核实)
*   **职业：** 某AI公司首席科学家
*   **当前城市：** (未提供)
*   **人物标签：** 自然探索者、求知者、哲思者、理论物理爱好者、实践者、**徒步爱好者**
*   **核心愿景：** 探索世界本质，追求知识，从自然中获取灵感，活出规律而美好的生活，**并通过实践拓展人生体验。**

---

### 主要人物刻画与关系网

*(记忆卡片中未提及具体人物，此部分暂无内容。)*

---

### 张三 时间主线与探索轨迹

**人生主线：** 从东北小村庄的自由童年启蒙，到高中时期对深奥物理学的热情探索，再到成年后对徒步等实践活动的投入，张三的人生轨迹贯穿着对自然的好奇、对知识的渴望以及持续的自我发现和体验拓展。

*   **童年与自然启蒙 (约1980年代 - 1995年前，辽宁葫芦岛小村庄)**
    *   **背景：** 出生于东北辽宁葫芦岛的一个小村庄，生活节奏简单、规律而美好，充满田园气息。
    *   **塑造/关键事件：**
        *   作为“山里的野孩子”，在自然中自由奔跑，体验无拘无束的童年，培养了对自然的亲近感。
        *   夜晚仰望“漫天繁星”的经历，是其早期对宇宙、对未知世界产生强烈好奇心和探索欲的萌芽，为其后来的科学兴趣奠定基础。
        *   简单的生活环境塑造了其对规律、美好和本真生活的初步认知。

*   **高中时期与知识探索 (约1995年左右，县城高中)**
    *   **背景：** 在资源相对有限的县城高中就读，学习重心主要集中在学科知识的掌握上。
    *   **塑造/关键事件：**
        *   在有限的学习条件下，对天文物理和理论物理产生了浓厚的兴趣，这显示出其强烈的内在驱动力和超越常规学习范围的求知欲。
        *   尽管环境可能不鼓励多元发展，但其对特定科学领域的专注，预示了其未来在深度探索上的潜力。

*   **成年后的实践与新爱好 (约2005年，云南虎跳峡)**
    *   **背景：** 在某个时间点，张三开始尝试新的实践活动，徒步成为其中一项。
    *   **塑造/关键事件：**
        *   **虎跳峡徒步初体验（2005年）：** 第一次徒步经历，在虎跳峡挑战陡峭的台阶。为了赶车，与朋友一起“猛爬，使劲地爬”，在身体极度疲惫的情况下仍坚持完成。
        *   这次经历不仅考验了其体能和毅力，更重要的是“开启了一个新的项目”，标志着其从纯粹的知识探索向身体力行的实践探索的延伸，丰富了其探索世界的方式。这次经历也展现了其在面对挑战时，目标导向和坚韧不拔的特质。

---

**张三 的“觉醒与探索”线索**

*   **自然作为第一启蒙：** 童年时期在山林间的自由奔跑和对星空的仰望，构成了其最初的“觉醒”，激发了对世界的好奇和探索的欲望。
*   **从感性到理性的飞跃：** 从对自然景象（漫天繁星）的直接感受，到高中时期对抽象的“天文物理和理论物理”的浓厚兴趣，展现了其思维从具象观察向理性、系统性思考的过渡。
*   **内在驱动的求知精神：** 即使在资源受限的高中阶段，其对物理学的热情依然强烈，表明其探索和学习的动力主要源于内在的兴趣和对知识本身的追求。
*   **对生活本质的思考：** 对童年“简单、规律、美好”生活的描述，暗示了其可能在追求高深科学的同时，也在追寻一种与自然和谐共处的生命状态。
*   **实践拓展与毅力展现：** 成年后的虎跳峡徒步，标志着其探索方式的拓展，从书本和理论走向了身体力行的实践。在极度疲惫下依然坚持完成目标，体现了其坚韧不拔的意志力，也揭示了其通过亲身体验来“开启新项目”的探索精神。

---

### 金句与细节描绘

#### 🌟 金句摘录

*   “当时我们都是山里的野孩子，没有什么特别的兴趣爱好，就在山里各种疯跑。”
*   “我小时候特别喜欢晚上看星星，那时的夜晚星星非常多，真的是那种突然就能看到漫天繁星的感觉。”
*   “我们所有的精力都放在学科的学习上。”
*   **“这次徒步对我来说，最大的收获就是开启了一个新的项目。”**

#### 🎨 优美细节描绘

*   “我出生在东北辽宁葫芦岛下面的一个小村庄. 小时候，那里的生活比较简单，人们日出而作，日落而息，生活节奏非常有规律，也非常美好。”
*   “那时的夜晚星星非常多，真的是那种突然就能看到漫天繁星的感觉。”
*   **“下去要一个小时，上来要一个多小时，而且都是很陡的台阶，非常累。”**
*   **“我和我的朋友为了赶上那趟车，就猛爬，使劲地爬。当时我们已经很累了，因为爬楼梯、爬那种台阶很累，但是为了赶上那趟车，我们还是一直在坚持跑。”**

---

### 图片素材备用

*   **出生东北_1:** http://39.96.146.47:8000/11.jpg
*   **高中的时候_1:** http://39.96.146.47:8000/11.jpg
*   **虎跳峡徒步_1:** (待补充，如果后续有图片链接)
"""
    params = {
            "model": "gemini-2.5-pro-preview-06-05",
            "messages": [{"role": "user", "content": text}],
            "temperature": 0.3,
        }
    response = await client.arequest(params = params)
    print(response)

asyncio.run(test_client_arequest_retry_function())

def test_client_request_tts():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))

    data = {
            "model": "tts-1",
            "input": "你好 世界",
            "voice": "alloy",
            "file_path": 'tests/resources/speech2.mp3',
        }
    response = client.request_tts(data)
    print(response)

async def test_client_arequest_tts():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))

    params = {
            "model": "tts-1",
            "input": "你好 世界 天气不错",
            "voice": "alloy",
            "file_path": 'tests/resources/speech.mp3',
        }
    response = await client.arequest_tts(params = params)
    print(response)

def test_client_request_asr():
    # 未通过
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))
    params = {
            "model": "whisper-1",
            "file_path": 'tests/resources/speech2.mp3',
        }
    response = client.request_asr(params = params)
    print(response)



def test_client_request_stream():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))

    data = {
            "model": "gemini-2.5-flash-preview-05-20-nothinking",
            "messages": [{"role": "user", "content": "你好"}],
            "temperature": 0.7,
            "stream": True,
        }
    response = client.request_stream(data)
    for i in response:
        print(i)

async def test_client_request_stream():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))

    data = {
            "model": "gemini-2.5-flash-preview-05-20-nothinking",
            "messages": [{"role": "user", "content": "你好"}],
            "temperature": 0.7,
            "stream": True,
        }
    response = client.arequest_stream(data)
    async for i in response:
        print(i)


def test_client_request_stream():
    client = OpenAIClient(api_key=os.getenv("BIANXIE_API_KEY"))

    params = {
            'model': 'gemini-2.5-flash-image-preview',
            'messages': [{'role': 'user', 'content': '绘制两个小孩的照片'}],
            "temperature": 0.7,
            'stream': True
        }
    response = client.request_image_stream(params,filename_prefix ="tests/resources/gemini_output" )
    for i in response:
        print(i)

def test_request_modal():
    pass



"""
模型适配器
"""
from abc import ABC
from llmada.client import OpenAIClient, ArkClient
from llmada.utils import image_to_base64, is_url_urllib
from typing import Iterator, Generator,AsyncIterator ,AsyncGenerator
import os




class ResourceExhaustedError(Exception):
    """Raised when a resource's quota has been exceeded."""

    def __init__(self, message="Token quota has been exhausted."):
        self.message = message
        super().__init__(self.message)


class ModelAdapter(ABC):
    """语言大模型的抽象类"""

    def __init__(self):
        self.api_key = None
        self.temperature = 0.7
        self.client = None
        self.model_name = None
        self.model_pool = []

    def set_model(self, model_name: str):
        """实例化以后用以修改调用的模型, 要求该模型存在于 model_pool 中

        Args:
            model_name (str): _description_
        """
        assert model_name in self.model_pool
        self.model_name = model_name

    def set_temperature(self, temperature: float):
        """用于设置模型的temperature

        Args:
            temperature (float): model temperature
        """
        self.temperature = temperature

    def get_model(self) -> list[str]:
        """获得当前的 model_pool

        Returns:
            list[str]: models
        """
        return self.model_pool

    def product(self, prompt: str) -> str:
        """子类必须实现的类, 用于和大模型做非流式交互

        Args:
            prompt (str): 提示词

        Raises:
            NotImplementedError: pass

        Returns:
            str: 大模型返回的结果
        """
        raise NotImplementedError("This method should be implemented by subclasses")

    def chat(self, messages: list) -> str:
        """子类必须实现的类, 用于和大模型做聊天交互

        Returns:
            str: 大模型返回的结果
        """
        raise NotImplementedError("This method should be implemented by subclasses")


class BianXieAdapter(ModelAdapter):
    """BianXie格式的适配器"""

    def __init__(
        self,
        model_name: str = "gemini-2.5-flash-preview-05-20-nothinking",
        api_key: str = None,
        api_base: str = "https://api.bianxie.ai/v1/chat/completions",
    ):
        """初始化
        正常情况下, 这两个参数都不需要传入, 而是会自动寻找环境变量,除非要临时改变api_key.
            api_base 不需要特别指定
        Args:
            api_key (str): API key for authentication.
            api_base (str): Base URL for the API endpoint.
        """
        super().__init__()
        self.client = OpenAIClient(
            api_key=api_key or os.getenv("BIANXIE_API_KEY"), api_base=api_base
        )
        self.model_name = model_name
        self.model_pool = [
            "gemini-2.5-flash-preview-05-20-nothinking",
            "gemini-2.5-flash-preview-04-17-nothinking",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0125",
            "gpt-4.1",
            "gpt-4.1-2025-04-14",
            "gpt-4.1-mini",
            "gpt-4.1-mini-2025-04-14",
            "gpt-4.1-nano",
            "gpt-4.1-nano-2025-04-14",
            "gpt-4o",
            "gpt-4o-2024-11-20",
            "gpt-4o-audio-preview",
            "gpt-4o-audio-preview-2024-10-01",
            "gpt-4o-audio-preview-2024-12-17",
            "gpt-4o-all",
            "gpt-4o-image",
            "gpt-4o-image-vip",
            "gpt-4o-mini",
            "gpt-4o-mini-2024-07-18",
            "gpt-4o-mini-audio-preview",
            "gpt-4o-mini-audio-preview-2024-12-17",
            "gpt-4o-mini-realtime-preview",
            "gpt-4o-mini-realtime-preview-2024-12-17",
            "gpt-4o-mini-search-preview",
            "gpt-4o-mini-search-preview-2025-03-11",
            "gpt-4o-realtime-preview",
            "gpt-4o-realtime-preview-2024-10-01",
            "gpt-4o-realtime-preview-2024-12-17",
            "gpt-4o-search-preview-2025-03-11",
            "gpt-4o-search-preview",
            "claude-3-5-haiku-20241022",
            "claude-3-5-haiku-latest",
            "claude-3-5-sonnet-20240620",
            "claude-3-5-sonnet-20241022",
            "claude-3-5-sonnet-20241022-all",
            "claude-3-5-sonnet-all",
            "claude-3-5-sonnet-latest",
            "claude-3-7-sonnet-20250219",
            "claude-3-7-sonnet-20250219-thinking",
            "claude-3-haiku-20240307",
            "claude-opus-4-20250514",
            "claude-opus-4-20250514-thinking",
            "claude-sonnet-4-20250514",
            "claude-sonnet-4-20250514-thinking",
            "coder-claude3.5-sonnet",
            "coder-claude3.7-sonnet",
            "gemini-2.0-flash",
            "gemini-2.0-flash-exp",
            "gemini-2.0-flash-exp-image-generation",
            "gemini-2.0-flash-thinking-exp",
            "gemini-2.0-flash-thinking-exp-01-21",
            "gemini-2.0-pro-exp-02-05",
            "gemini-2.5-flash-preview-04-17",
            "gemini-2.5-flash-preview-04-17-thinking",
            "gemini-2.5-pro-exp-03-25",
            "gemini-2.5-pro-preview-03-25",
            "gemini-2.5-pro-preview-03-25-thinking",
            "gemini-2.5-pro-preview-05-06",
            "gemini-2.5-pro-preview-05-06-thinking",
            "gemini-2.5-pro-preview-06-05",
            "gemini-2.5-pro-preview-06-05-thinking",
            "deepseek-ai/DeepSeek-R1",
            "deepseek-ai/DeepSeek-V3",
            "deepseek-chat",
            "deepseek-r1",
            "deepseek-r1-250528",
            "deepseek-reasoner",
            "grok-3",
            "grok-3-beta",
            "grok-3-deepsearch",
            "grok-3-mini-beta",
            "grok-3-fast-beta",
            "grok-3-mini-fast-beta",
            "grok-3-reasoner",
            "grok-beta",
            "grok-vision-beta",
            "o1-mini",
            "o1-mini-2024-09-12",
            "o3-mini",
            "o3-mini-2025-01-31",
            "o3-mini-all",
            "o3-mini-high",
            "o3-mini-low",
            "o3-mini-medium",
            "o4-mini",
            "o4-mini-2025-04-16",
            "o4-mini-high",
            "o4-mini-medium",
            "o4-mini-low",
            "text-embedding-ada-002",
            "text-embedding-3-small",
            "text-embedding-3-large",
            "deepseek-v3",
            "gpt-5",
        ]

    def product(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
        }
        response = self.client.request(data)
        return self._deal_response(response=response)

    async def aproduct(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
        }
        response = await self.client.arequest(data)
        return self._deal_response(response=response)

    def product_stream(self, prompt: str) -> Iterator[str]:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
            "stream": True,
        }
        if not self._assert_prompt(prompt):
            yield ""
        else:
            result_stream = self.client.request_stream(data)
            for i in result_stream:
                yield i

    async def aproduct_stream(self, prompt: str) -> AsyncIterator[str]:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
            "stream": True,
        }
        async for i in self.client.arequest_stream(data):
            yield i

    def product_by_dict(self, params: dict):
        response = self.client.request(params)
        return response

    async def aproduct_by_dict(self, params: dict):
        response = await self.client.arequest(params)
        return response

    def _assert_prompt(self, prompt):
        try:
            assert prompt != ""
            return True
        except AssertionError:
            return False

    def _deal_response(self, response):
        """
        处理事件的相应
        """
        choices = response.get("choices")
        if choices:
            content = choices[0].get("message").get("content")
            return content
        else:
            raise ResourceExhaustedError(f"{response.get('error')}")

    def product_image_stream(self, prompt: str, image_path: str) -> Iterator[str]:
        """Generate a response from the model based on a single prompt.
        # TODO

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.

        image_path : 文件路径 或者 url
        """
        if is_url_urllib(image_path):
            image_content = image_path
        else:
            image_base64 = image_to_base64(image_path)
            image_content = f"data:image/jpeg;base64,{image_base64}"

        assert self.model_name in [
            "gemini-2.5-flash-image-preview",
            "gemini-2.0-flash-preview-image-generation",
        ]
        data = {
            "model": self.model_name,
            "messages": [
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": prompt},
                        {"type": "image_url", "image_url": {"url": image_content}},
                    ],
                }
            ],
            "temperature": self.temperature,
            "stream": True,
        }

        if not self._assert_prompt(prompt):
            yield ""
        else:
            result_stream = self.client.request_image_stream(data)
            for i in result_stream:
                yield i

class ArkAdapter(ModelAdapter):
    def __init__(
        self,
        model_name: str = "doubao-1-5-pro-256k-250115",
        api_key: str = None,
        api_base: str = None,
    ):
        """初始化

        Args:
            api_key (str): API key for authentication.
            api_base (str): Base URL for the API endpoint.
        """
        super().__init__()

        self.client = ArkClient(api_key=api_key or os.getenv("ARK_API_KEY"))
        self.model_pool = ["doubao-1-5-pro-256k-250115","doubao-1-5-pro-32k-250115"]
        self.model_name = model_name

    def product(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
        }
        response = self.client.product(data)

        return response.choices[0].message.content
        # return self._deal_response(response=response)

    async def aproduct(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
        }
        response = await self.client.aproduct(data)

        return response.choices[0].message.content
        # return self._deal_response(response=response)


    def product_stream(self, prompt: str):
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
            "stream": True,
        }
        response = self.client.product_stream(data)
        for chunk in response:
            yield chunk

    async def aproduct_stream(self, prompt: str):
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            "model": self.model_name,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": self.temperature,
            "stream": True,
        }
        response = self.client.aproduct_stream(data)
        async for chunk in response:
            yield chunk
    
    async def tts(self,text: str,filename:str, encoding="wav",voice_type = "zh_female_yuanqinvyou_moon_bigtts"):
        """
        
        """
        await self.client.generate2file(text = text,
                                  filename = filename,
                                  encoding=encoding,
                                  voice_type = voice_type)






from urllib.parse import urlparse
import base64
import re
from llmada.log import Log
logger = Log.logger

def save_base64_image(markdown_line, filename_prefix="gemini_output"):
    """
    提取 base64 字符串
    """

    match = re.search(r"!\[.*?\]\(data:image/(png|jpeg);base64,(.*?)\)", markdown_line)
    if match:
        image_format = match.group(1)  # png 或 jpeg
        base64_data = match.group(2)

        # 解码并保存
        image_data = base64.b64decode(base64_data)
        filename = f"{filename_prefix}.{image_format}"
        with open(f"{filename}", "wb") as f:
            f.write(image_data)
        logger.info(f"图片已保存为 {filename}")
    else:
        print("未找到合法的 base64 图片数据")


def image_to_base64(image_path:str) ->str:
    """
    以二进制方式读取图片文件
    """
    with open(image_path, "rb") as image_file:
        # 将图像文件内容读取到变量中
        image_data = image_file.read()
        # 使用base64模块进行编码
        base64_encoded_data = base64.b64encode(image_data)
        # 将编码后的数据转换为字符串
        base64_encoded_str = base64_encoded_data.decode("utf-8")
        return base64_encoded_str


def is_url_urllib(url_string: str):
    """
    检查 scheme 和 netloc 是否存在，并且 netloc 不为空
    通常，一个有效的 URL 应该有 scheme (如 http, https) 和 netloc (如 www.example.com)
    """
    try:
        result = urlparse(url_string)
        return all([result.scheme, result.netloc])
    except Exception:
        return False



import io
import logging
import struct
from dataclasses import dataclass
from enum import IntEnum
from typing import Callable, List

import websockets

logger = logging.getLogger(__name__)


class MsgType(IntEnum):
    """Message type enumeration"""

    Invalid = 0
    FullClientRequest = 0b1
    AudioOnlyClient = 0b10
    FullServerResponse = 0b1001
    AudioOnlyServer = 0b1011
    FrontEndResultServer = 0b1100
    Error = 0b1111

    # Alias
    ServerACK = AudioOnlyServer

    def __str__(self) -> str:
        return self.name if self.name else f"MsgType({self.value})"


class MsgTypeFlagBits(IntEnum):
    """Message type flag bits"""

    NoSeq = 0  # Non-terminal packet with no sequence
    PositiveSeq = 0b1  # Non-terminal packet with sequence > 0
    LastNoSeq = 0b10  # Last packet with no sequence
    NegativeSeq = 0b11  # Last packet with sequence < 0
    WithEvent = 0b100  # Payload contains event number (int32)


class VersionBits(IntEnum):
    """Version bits"""

    Version1 = 1
    Version2 = 2
    Version3 = 3
    Version4 = 4


class HeaderSizeBits(IntEnum):
    """Header size bits"""

    HeaderSize4 = 1
    HeaderSize8 = 2
    HeaderSize12 = 3
    HeaderSize16 = 4


class SerializationBits(IntEnum):
    """Serialization method bits"""

    Raw = 0
    JSON = 0b1
    Thrift = 0b11
    Custom = 0b1111


class CompressionBits(IntEnum):
    """Compression method bits"""

    None_ = 0
    Gzip = 0b1
    Custom = 0b1111


class EventType(IntEnum):
    """Event type enumeration"""

    None_ = 0  # Default event

    # 1 ~ 49 Upstream Connection events
    StartConnection = 1
    StartTask = 1  # Alias of StartConnection
    FinishConnection = 2
    FinishTask = 2  # Alias of FinishConnection

    # 50 ~ 99 Downstream Connection events
    ConnectionStarted = 50  # Connection established successfully
    TaskStarted = 50  # Alias of ConnectionStarted
    ConnectionFailed = 51  # Connection failed (possibly due to authentication failure)
    TaskFailed = 51  # Alias of ConnectionFailed
    ConnectionFinished = 52  # Connection ended
    TaskFinished = 52  # Alias of ConnectionFinished

    # 100 ~ 149 Upstream Session events
    StartSession = 100
    CancelSession = 101
    FinishSession = 102

    # 150 ~ 199 Downstream Session events
    SessionStarted = 150
    SessionCanceled = 151
    SessionFinished = 152
    SessionFailed = 153
    UsageResponse = 154  # Usage response
    ChargeData = 154  # Alias of UsageResponse

    # 200 ~ 249 Upstream general events
    TaskRequest = 200
    UpdateConfig = 201

    # 250 ~ 299 Downstream general events
    AudioMuted = 250

    # 300 ~ 349 Upstream TTS events
    SayHello = 300

    # 350 ~ 399 Downstream TTS events
    TTSSentenceStart = 350
    TTSSentenceEnd = 351
    TTSResponse = 352
    TTSEnded = 359
    PodcastRoundStart = 360
    PodcastRoundResponse = 361
    PodcastRoundEnd = 362

    # 450 ~ 499 Downstream ASR events
    ASRInfo = 450
    ASRResponse = 451
    ASREnded = 459

    # 500 ~ 549 Upstream dialogue events
    ChatTTSText = 500  # (Ground-Truth-Alignment) text for speech synthesis

    # 550 ~ 599 Downstream dialogue events
    ChatResponse = 550
    ChatEnded = 559

    # 650 ~ 699 Downstream dialogue events
    # Events for source (original) language subtitle
    SourceSubtitleStart = 650
    SourceSubtitleResponse = 651
    SourceSubtitleEnd = 652
    # Events for target (translation) language subtitle
    TranslationSubtitleStart = 653
    TranslationSubtitleResponse = 654
    TranslationSubtitleEnd = 655

    def __str__(self) -> str:
        return self.name if self.name else f"EventType({self.value})"


@dataclass
class Message:
    """Message object

    Message format:
    0                 1                 2                 3
    | 0 1 2 3 4 5 6 7 | 0 1 2 3 4 5 6 7 | 0 1 2 3 4 5 6 7 | 0 1 2 3 4 5 6 7 |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |    Version      |   Header Size   |     Msg Type    |      Flags      |
    |   (4 bits)      |    (4 bits)     |     (4 bits)    |     (4 bits)    |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    | Serialization   |   Compression   |           Reserved                |
    |   (4 bits)      |    (4 bits)     |           (8 bits)                |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                                                                       |
    |                   Optional Header Extensions                          |
    |                     (if Header Size > 1)                              |
    |                                                                       |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |                                                                       |
    |                           Payload                                     |
    |                      (variable length)                                |
    |                                                                       |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    """

    version: VersionBits = VersionBits.Version1
    header_size: HeaderSizeBits = HeaderSizeBits.HeaderSize4
    type: MsgType = MsgType.Invalid
    flag: MsgTypeFlagBits = MsgTypeFlagBits.NoSeq
    serialization: SerializationBits = SerializationBits.JSON
    compression: CompressionBits = CompressionBits.None_

    event: EventType = EventType.None_
    session_id: str = ""
    connect_id: str = ""
    sequence: int = 0
    error_code: int = 0

    payload: bytes = b""

    @classmethod
    def from_bytes(cls, data: bytes) -> "Message":
        """Create message object from bytes"""
        if len(data) < 3:
            raise ValueError(
                f"Data too short: expected at least 3 bytes, got {len(data)}"
            )

        type_and_flag = data[1]
        msg_type = MsgType(type_and_flag >> 4)
        flag = MsgTypeFlagBits(type_and_flag & 0b00001111)

        msg = cls(type=msg_type, flag=flag)
        msg.unmarshal(data)
        return msg

    def marshal(self) -> bytes:
        """Serialize message to bytes"""
        buffer = io.BytesIO()

        # Write header
        header = [
            (self.version << 4) | self.header_size,
            (self.type << 4) | self.flag,
            (self.serialization << 4) | self.compression,
        ]

        header_size = 4 * self.header_size
        if padding := header_size - len(header):
            header.extend([0] * padding)

        buffer.write(bytes(header))

        # Write other fields
        writers = self._get_writers()
        for writer in writers:
            writer(buffer)

        return buffer.getvalue()

    def unmarshal(self, data: bytes) -> None:
        """Deserialize message from bytes"""
        buffer = io.BytesIO(data)

        # Read version and header size
        version_and_header_size = buffer.read(1)[0]
        self.version = VersionBits(version_and_header_size >> 4)
        self.header_size = HeaderSizeBits(version_and_header_size & 0b00001111)

        # Skip second byte
        buffer.read(1)

        # Read serialization and compression methods
        serialization_compression = buffer.read(1)[0]
        self.serialization = SerializationBits(serialization_compression >> 4)
        self.compression = CompressionBits(serialization_compression & 0b00001111)

        # Skip header padding
        header_size = 4 * self.header_size
        read_size = 3
        if padding_size := header_size - read_size:
            buffer.read(padding_size)

        # Read other fields
        readers = self._get_readers()
        for reader in readers:
            reader(buffer)

        # Check for remaining data
        remaining = buffer.read()
        if remaining:
            raise ValueError(f"Unexpected data after message: {remaining}")

    def _get_writers(self) -> List[Callable[[io.BytesIO], None]]:
        """Get list of writer functions"""
        writers = []

        if self.flag == MsgTypeFlagBits.WithEvent:
            writers.extend([self._write_event, self._write_session_id])

        if self.type in [
            MsgType.FullClientRequest,
            MsgType.FullServerResponse,
            MsgType.FrontEndResultServer,
            MsgType.AudioOnlyClient,
            MsgType.AudioOnlyServer,
        ]:
            if self.flag in [MsgTypeFlagBits.PositiveSeq, MsgTypeFlagBits.NegativeSeq]:
                writers.append(self._write_sequence)
        elif self.type == MsgType.Error:
            writers.append(self._write_error_code)
        else:
            raise ValueError(f"Unsupported message type: {self.type}")

        writers.append(self._write_payload)
        return writers

    def _get_readers(self) -> List[Callable[[io.BytesIO], None]]:
        """Get list of reader functions"""
        readers = []

        if self.type in [
            MsgType.FullClientRequest,
            MsgType.FullServerResponse,
            MsgType.FrontEndResultServer,
            MsgType.AudioOnlyClient,
            MsgType.AudioOnlyServer,
        ]:
            if self.flag in [MsgTypeFlagBits.PositiveSeq, MsgTypeFlagBits.NegativeSeq]:
                readers.append(self._read_sequence)
        elif self.type == MsgType.Error:
            readers.append(self._read_error_code)
        else:
            raise ValueError(f"Unsupported message type: {self.type}")

        if self.flag == MsgTypeFlagBits.WithEvent:
            readers.extend(
                [self._read_event, self._read_session_id, self._read_connect_id]
            )

        readers.append(self._read_payload)
        return readers

    def _write_event(self, buffer: io.BytesIO) -> None:
        """Write event"""
        buffer.write(struct.pack(">i", self.event))

    def _write_session_id(self, buffer: io.BytesIO) -> None:
        """Write session ID"""
        if self.event in [
            EventType.StartConnection,
            EventType.FinishConnection,
            EventType.ConnectionStarted,
            EventType.ConnectionFailed,
        ]:
            return

        session_id_bytes = self.session_id.encode("utf-8")
        size = len(session_id_bytes)
        if size > 0xFFFFFFFF:
            raise ValueError(f"Session ID size ({size}) exceeds max(uint32)")

        buffer.write(struct.pack(">I", size))
        if size > 0:
            buffer.write(session_id_bytes)

    def _write_sequence(self, buffer: io.BytesIO) -> None:
        """Write sequence number"""
        buffer.write(struct.pack(">i", self.sequence))

    def _write_error_code(self, buffer: io.BytesIO) -> None:
        """Write error code"""
        buffer.write(struct.pack(">I", self.error_code))

    def _write_payload(self, buffer: io.BytesIO) -> None:
        """Write payload"""
        size = len(self.payload)
        if size > 0xFFFFFFFF:
            raise ValueError(f"Payload size ({size}) exceeds max(uint32)")

        buffer.write(struct.pack(">I", size))
        buffer.write(self.payload)

    def _read_event(self, buffer: io.BytesIO) -> None:
        """Read event"""
        event_bytes = buffer.read(4)
        if event_bytes:
            self.event = EventType(struct.unpack(">i", event_bytes)[0])

    def _read_session_id(self, buffer: io.BytesIO) -> None:
        """Read session ID"""
        if self.event in [
            EventType.StartConnection,
            EventType.FinishConnection,
            EventType.ConnectionStarted,
            EventType.ConnectionFailed,
            EventType.ConnectionFinished,
        ]:
            return

        size_bytes = buffer.read(4)
        if size_bytes:
            size = struct.unpack(">I", size_bytes)[0]
            if size > 0:
                session_id_bytes = buffer.read(size)
                if len(session_id_bytes) == size:
                    self.session_id = session_id_bytes.decode("utf-8")

    def _read_connect_id(self, buffer: io.BytesIO) -> None:
        """Read connection ID"""
        if self.event in [
            EventType.ConnectionStarted,
            EventType.ConnectionFailed,
            EventType.ConnectionFinished,
        ]:
            size_bytes = buffer.read(4)
            if size_bytes:
                size = struct.unpack(">I", size_bytes)[0]
                if size > 0:
                    self.connect_id = buffer.read(size).decode("utf-8")

    def _read_sequence(self, buffer: io.BytesIO) -> None:
        """Read sequence number"""
        sequence_bytes = buffer.read(4)
        if sequence_bytes:
            self.sequence = struct.unpack(">i", sequence_bytes)[0]

    def _read_error_code(self, buffer: io.BytesIO) -> None:
        """Read error code"""
        error_code_bytes = buffer.read(4)
        if error_code_bytes:
            self.error_code = struct.unpack(">I", error_code_bytes)[0]

    def _read_payload(self, buffer: io.BytesIO) -> None:
        """Read payload"""
        size_bytes = buffer.read(4)
        if size_bytes:
            size = struct.unpack(">I", size_bytes)[0]
            if size > 0:
                self.payload = buffer.read(size)

    def __str__(self) -> str:
        """String representation"""
        if self.type in [MsgType.AudioOnlyServer, MsgType.AudioOnlyClient]:
            if self.flag in [MsgTypeFlagBits.PositiveSeq, MsgTypeFlagBits.NegativeSeq]:
                return f"MsgType: {self.type}, EventType:{self.event}, Sequence: {self.sequence}, PayloadSize: {len(self.payload)}"
            return f"MsgType: {self.type}, EventType:{self.event}, PayloadSize: {len(self.payload)}"
        elif self.type == MsgType.Error:
            return f"MsgType: {self.type}, EventType:{self.event}, ErrorCode: {self.error_code}, Payload: {self.payload.decode('utf-8', 'ignore')}"
        else:
            if self.flag in [MsgTypeFlagBits.PositiveSeq, MsgTypeFlagBits.NegativeSeq]:
                return f"MsgType: {self.type}, EventType:{self.event}, Sequence: {self.sequence}, Payload: {self.payload.decode('utf-8', 'ignore')}"
            return f"MsgType: {self.type}, EventType:{self.event}, Payload: {self.payload.decode('utf-8', 'ignore')}"


async def receive_message(websocket) -> Message:
    """Receive message from websocket"""
    try:
        data = await websocket.recv()
        if isinstance(data, str):
            raise ValueError(f"Unexpected text message: {data}")
        elif isinstance(data, bytes):
            msg = Message.from_bytes(data)
            logger.info(f"Received: {msg}")
            return msg
        else:
            raise ValueError(f"Unexpected message type: {type(data)}")
    except Exception as e:
        logger.error(f"Failed to receive message: {e}")
        raise


async def wait_for_event(
    websocket,
    msg_type: MsgType,
    event_type: EventType,
) -> Message:
    """Wait for specific event"""
    while True:
        msg = await receive_message(websocket)
        if msg.type != msg_type or msg.event != event_type:
            raise ValueError(f"Unexpected message: {msg}")
        if msg.type == msg_type and msg.event == event_type:
            return msg


async def full_client_request(
    websocket, payload: bytes
) -> None:
    """Send full client message"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.NoSeq)
    msg.payload = payload
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())


async def audio_only_client(
    websocket, payload: bytes, flag: MsgTypeFlagBits
) -> None:
    """Send audio-only client message"""
    msg = Message(type=MsgType.AudioOnlyClient, flag=flag)
    msg.payload = payload
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())


async def start_connection(websocket) -> None:
    """Start connection"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.WithEvent)
    msg.event = EventType.StartConnection
    msg.payload = b"{}"
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())


async def finish_connection(websocket) -> None:
    """Finish connection"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.WithEvent)
    msg.event = EventType.FinishConnection
    msg.payload = b"{}"
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())



async def start_session(
    websocket, payload: bytes, session_id: str
) -> None:
    """Start session"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.WithEvent)
    msg.event = EventType.StartSession
    msg.session_id = session_id
    msg.payload = payload
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())


async def finish_session(
    websocket, session_id: str
) -> None:
    """Finish session"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.WithEvent)
    msg.event = EventType.FinishSession
    msg.session_id = session_id
    msg.payload = b"{}"
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())


async def cancel_session(
    websocket, session_id: str
) -> None:
    """Cancel session"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.WithEvent)
    msg.event = EventType.CancelSession
    msg.session_id = session_id
    msg.payload = b"{}"
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())
# : websockets.client.WebSocketClientProtocol

async def task_request(
    websocket, payload: bytes, session_id: str
) -> None:
    """Send task request"""
    msg = Message(type=MsgType.FullClientRequest, flag=MsgTypeFlagBits.WithEvent)
    msg.event = EventType.TaskRequest
    msg.session_id = session_id
    msg.payload = payload
    logger.info(f"Sending: {msg}")
    await websocket.send(msg.marshal())
